// ###########################################################
// CONSTANTS, GLOBAL VARIABLES

var faunaFields = [];
// ###########################################################
// SET UP TABULATOR TABLE(S)

// ###########################################################
// RUN ON PAGE LOAD
$(document).ready(function() {

    getUniques();
    
    setTimeout(function() {
        $("#sighting").hide('slow');
        $("#reference").hide('slow');
    },2000);
    
    // submission type check box listeners

    var sighting = document.querySelector("#sightingCheck");
    sighting.addEventListener( 'change', function() {
        if(this.checked) {
            $("#sighting").show('slow');
        } else {
            $("#sighting").hide('slow');
        }
    });

    var reference = document.querySelector("#referenceCheck");
    reference.addEventListener( 'change', function() {
        if(this.checked) {
            $("#reference").show('slow');
        } else {
            $("#reference").hide('slow');
        }
    });

});


// ###########################################################
// FUNCTIONS

function getUniques() {
    $.get( `${APIpath}uniques?category=fauna`, function( result ) {
        var data = JSON.parse(result);
        faunaFields = Object.keys(data);
        var optionsHTML = '';

        faunaFields.forEach(field => {

            if(field == 'Link') return; // skip Link field

            optionsHTML = '<option value=""></option><option value=""></option>';
            data[field].forEach(element => {
                optionsHTML += `<option value="${element}">${element}</option>`;
            });

            $('#select'+field).html(optionsHTML);

            var placeHolderText = 'Select, or type a new value';
            if(field == 'ScientificName') placeHolderText = 'Make sure you type in a unique value';

            $('#select'+field).select2({
                tags: true,
                width: 'element',
                theme: "classic",
                placeholder: placeHolderText,
                allowClear: true,
            });
        });

    });

}

function faunaSubmit() {
    $('#faunaStatus').html('<div class="alert alert-warning">Processing, please wait..</div>');

    var package = {};
    package['submission'] = {};
    faunaFields.forEach(field => {
        package['submission'][field] = $('#select'+field).val().trim();
    });
    // add link field:
    package['submission']['Link'] = $('#link').val().trim();

    package['Name'] = $('#name').val().trim();
    package['Email'] = $('#email').val().trim();
    package['Number'] = $('#number').val().trim().replace(/[^0-9+-]/g, "");
    $('#number').val(package['Number']);

    // ##################
    // VALIDATION
    //name:
    if (!package['submission']['LocalName'].length && !package['submission']['ScientificName'].length) {
        $('#faunaStatus').html('<div class="alert alert-danger">At least Local Name or Scientific Name must be provided to make a valid submission.</div>'); 
        shakeIt('row1');
        return; 
    }

    // submission Type: location or source checked?
    if(!$('#sightingCheck').is(':checked') && !$('#referenceCheck').is(':checked')) {
        $('#faunaStatus').html('<div class="alert alert-danger">It needs to be either a Sighting or a Reference.</div>'); 
        shakeIt('row3'); return; 
    }

    // location if checked
    if( $('#sightingCheck').is(':checked') && !package['submission']['Location'].length) {
        $('#faunaStatus').html('<div class="alert alert-danger">You have to enter a Location.</div>'); 
        shakeIt('row3'); return;
    }
    // source if checked
    if( $('#referenceCheck').is(':checked') && !package['submission']['Source'].length) {
        $('#faunaStatus').html('<div class="alert alert-danger">You have to enter a Source.</div>'); 
        shakeIt('row3'); return;
    }

    if(!package['Name'].length) { 
        $('#faunaStatus').html('<div class="alert alert-danger">Please enter your Name</div>'); 
        shakeIt('name'); return; 
    }
    if(package['Number'].length < 10) { 
        $('#faunaStatus').html('<div class="alert alert-danger">Please enter your Number in full.</div>'); 
        shakeIt('number'); return; 
    }

    // ##################
    

    console.log(JSON.stringify(package));

    request = $.ajax({
		url: `${APIpath}addFauna`,
		type: "post",
		data: JSON.stringify(package),
		success: function(result){
            console.log('addFauna POST call successful.');
            clearForm();
			$('#faunaStatus').html('<div class="alert alert-success">' + result + '</div>');	
        },
        failure: function(result){
            console.log('addFauna POST call FAILED.');
            console.log('result:',result);
        }	
	}); 
}

function clearForm() {
    faunaFields.forEach(field => {
        $('#select'+field).val(null).trigger('change'); // from https://select2.org/programmatic-control/add-select-clear-items
    });
    // clear checkboxes
    $("#sightingCheck").prop("checked", false);
    $("#sighting").hide('slow');
    $("#referenceCheck").prop("checked", false);
    $("#reference").hide('slow');
}
