'''
Peoples Biodiversity Register App
start date: 22.12.18
author: Nikhil VJ, http://nikhilvj.co.in
Created for PMC Environment Office
'''

print('\nPeoples Biodiversity Register')
print('Starting up the program, loading dependencies, please wait...\n\n')

import datetime
import time
import json
import os
import signal  # for catching Ctrl+C and exiting gracefully.
import sys
import webbrowser
from collections import OrderedDict
from random import randint # for generating random num

import pandas as pd
import tornado.ioloop
import tornado.web

# setting constants
root = os.path.dirname(__file__) # needed for tornado
logFolder = os.path.join(root,'logs/')
dataFolder = os.path.join(root,'../biodiversity-register-data/')

floraCSV = os.path.join(dataFolder,'Flora.csv')
faunaCSV = os.path.join(dataFolder,'Fauna.csv')
submittersFile = os.path.join(dataFolder,'submitters.csv')

# create folders if they don't exist
for folder in [logFolder, dataFolder]:
    if not os.path.exists(folder):
        os.makedirs(folder)

floraFields = ['LocalName','ScientificName','TypeOfPlants','Habitat','FloweringSeason','Remarks','Location','Link','Source','YearOfPublication']
faunaFields = ['LocalName', 'ScientificName', 'TypeOfFauna', 'Habitat', 'Family', 'Abundance', 'Remarks', 'Location', 'Link', 'Source', 'YearOfPublication']
submitterFields = ['UserID','Name','Number','Email']

print('Loaded dependencies, starting program.')

###########################
# Define all API call classes here

class uniques(tornado.web.RequestHandler):
    def get(self):
        # /API/uniques?category=fauna
        start = time.time()
        category = self.get_argument('category',default='flora')

        if category == 'flora':
            container = loadCSVuniques()
        else:
            container = loadCSVuniques('Fauna')
        
        self.write(json.dumps(container))
        end = time.time()
        logmessage("uniques GET call took {} seconds.".format(round(end-start,2)))

class addFlora(tornado.web.RequestHandler):
    def post(self):
        # /API/addFlora
        start = time.time()
        
        package = json.loads( self.request.body.decode('UTF-8') )
        returnMessage = submit2CSV(package)
        
        self.write(returnMessage)
        end = time.time()
        logmessage("addFlora POST call took {} seconds.".format(round(end-start,2)))



class addFauna(tornado.web.RequestHandler):
    def post(self):
        # /API/addFauna
        start = time.time()
        
        package = json.loads( self.request.body.decode('UTF-8') )
        returnMessage = submit2CSV(package,category='Fauna')
        
        self.write(returnMessage)
        end = time.time()
        logmessage("addFauna POST call took {} seconds.".format(round(end-start,2)))

class table(tornado.web.RequestHandler):
    def get(self):
        # /API/table?category=fauna
        start = time.time()
        category = self.get_argument('category',default='flora')

        if category == 'flora':
            csvFile = floraCSV
            columns = floraFields[:-4].copy() 
            # shit, appending to columns later on was changing floraFields too!
        else:
            csvFile = faunaCSV
            columns = faunaFields[:-4].copy()
        
        logmessage(csvFile)
        # columns = columns[:-2] # naah, let'em have it all.
        columns.append('sr')
        columns.append('UserID')
        columns.append('Status')
        logmessage(columns)
        # order is irrelevant as the tabulator tables are deciding the final order.
        df = pd.read_csv(csvFile,dtype=str).fillna('')[columns]
        container = df.to_dict(orient='records', into=OrderedDict)
        
        self.write(json.dumps(container))
        end = time.time()
        logmessage("table GET call took {} seconds.".format(round(end-start,2)))

###################
# List API calls and the class they will be directed to, here
def make_app():
    return tornado.web.Application([
        #(r"/API/data", APIHandler),
        (r"/API/uniques", uniques),
        (r"/API/addFlora", addFlora),
        (r"/API/addFauna", addFauna),
        (r"/API/table", table),
        #(r"/API/loadRouteCSV", loadRouteCSV),
        (r"/(.*)", tornado.web.StaticFileHandler, {"path": root, "default_filename": "index.html"})
    ])


###################
# Done with API calls defining. 

####################
# Functions

# for catching Ctrl+C and exiting gracefully. From https://nattster.wordpress.com/2013/06/05/catch-kill-signal-in-python/
def signal_term_handler(signal, frame):
    # to do: Make this work in windows, ra!
    logmessage('\nClosing Program.\nThank you for using this program.\n')
    sys.exit(0)

def loadCSVuniques(category='Flora'):

    if category == 'Flora':
        csvFile = floraCSV
        fields = floraFields
    else:
        csvFile = faunaCSV
        fields = faunaFields
    
    df = pd.read_csv(csvFile,dtype=str).fillna('')
    # logmessage(df.sample(5))
    fields_nolink = [x for x in fields if x != 'Link'] # remove Link field
    collector = OrderedDict()
    for field in fields_nolink:
        uniq = df[field].unique().tolist()
        uniq.sort()
        if '' in uniq: uniq.remove('')
        collector[field] = uniq
    
    # to do : get rid of lat-long values from location field. Later.
    
    return collector



def submit2CSV(package, category='Flora'):
    if category == 'Flora':
        csvFile = floraCSV
        fields = floraFields
    else:
        csvFile = faunaCSV
        fields = faunaFields

    for key,value in package.get('submission',{}).items():
        package['submission'][key] = str(value).strip()
    # clean submitters info also
    package['Name'] = str(package.get('Name','')).strip().title()
    package['Email'] = str(package.get('Email','')).strip()
    package['Number'] = str(package.get('Number','')).strip()

    print('Email:',package['Email'])
    print('Number:',package['Number'])
    # processing submitter info
    pplDF = pd.read_csv(submittersFile, dtype=str).fillna('')
    
    # search pplDF
    oldUsers = pd.DataFrame()
    if package['Number'] != '':
        oldUsers = pplDF[ pplDF['Number'].str.strip() == package['Number']].copy().reset_index(drop=True)

    else:
        if package['Email'] != '':
            oldUsers = pplDF[ pplDF['Email'].str.strip() == package['Email']].copy().reset_index(drop=True)


    if len(oldUsers):
        package['UserID'] = oldUsers['UserID'].iloc[0]

    else:
        # create a new UserID and append to submitters csv 
        datestamp = '{:%Y.%m.%d}'.format(datetime.datetime.now())
        initial = package.get('Name','X')[0].upper()
        rand4 = str( randint(1000, 9999) )
        package['UserID'] = '{}.{}.{}'.format(datestamp,initial,rand4)

        addDict = {
            'UserID': package['UserID'],
            'Name' : package['Name'],
            'Number' : package['Number'],
            'Email' : package['Email']
        } # crap, server isn't adhering to the order! have to specify columns order at write command
        
        addIDdf = pd.DataFrame([ addDict ])  
        logmessage('New submitter entry:',addDict)
        # logmessage(addIDdf.columns.tolist())

        addIDdf.to_csv(submittersFile, columns=submitterFields, mode='a', index=False, header=False)

    logmessage('UserID:',package['UserID'])

    # now add the sumbitted data to the csv

    df = pd.read_csv(csvFile, dtype=str).fillna('')
    serialNum = len(df)+1

    submission = OrderedDict({ 'sr':serialNum })
    for field in fields:
        submission[field] = package.get('submission',{}).get(field,'')
    
    submission['UserID'] = package.get('UserID','')

    # intervention: check for duplicates
    # 3 modes: local + scientific, local-only, scientific-only.
    # make all to lowercase for comparison
    # and consider zapping extra chars too.
    # we can mess with df - we're not writing it back.
    
    LocalNameZap = submission['LocalName'].lower().strip()
    ScientificNameZap = submission['ScientificName'].lower().strip()

    #zap em!
    df['LocalName'] = df['LocalName'].str.lower().str.strip()
    df['ScientificName'] = df['ScientificName'].str.lower().str.strip()

    # 12.1.19 : to do: mtch only by ScientificName and say if that's repeating. If that is missing, then check for LocalName matching.
    dfMatch = pd.DataFrame()

    if ScientificNameZap:
        dfMatch = df[ df['ScientificName'] == ScientificNameZap ]
    elif LocalNameZap:
        dfMatch = df[ df['LocalName'] == LocalNameZap ]
    
    '''
    # consecutive filter.. for second round all the available rows are matching localname so the "AND" is achieved.
    if ScientificNameZap:
        df = df[ df['ScientificName'] == ScientificNameZap ]
        
    if LocalNameZap or ScientificNameZap:
        if len(df):
    '''
    if len(dfMatch):
        logmessage('duplicate(s) found in rows: ')
        logmessage(', '.join(dfMatch.sr.tolist()))
        returnMessage = 'Thank you for your submission. Since we already have an entry with the same Scientific Name / Local Name in row(s) <i>{}</i>, we are recording it as a "Repeat" submission. We will still keep your data for later review.<br>'.format(', '.join(dfMatch.sr.tolist()) )
        submission['Status'] = 'Repeat'

    else:
        logmessage('this is a unique submission.')
        returnMessage = 'Thank you for your submission.<br>'
        submission['Status'] = 'Submitted'
    
    logmessage('Adding this row to',csvFile)
    logmessage(submission)
    
    pd.DataFrame([submission]).to_csv(csvFile, mode='a', index=False, header=False)

    returnMessage += 'Please copy and save the details below for your records.<br>'
    for field in fields:
        returnMessage += '{} : <b>{}</b><br>'.format(field,submission[field])

    returnMessage += '<br>Here is your auto-generated user-id: <b>{}</b> <br>You can use this to track your submissions in the final data.<br>'.format(package['UserID'])
    return returnMessage


def logmessage( *content ):
	timestamp = '{:%Y-%b-%d %H:%M:%S} :'.format(datetime.datetime.now())
	# from https://stackoverflow.com/a/26455617/4355695
	line = ' '.join(str(x) for x in list(content))
	# str(x) for x in list(content) : handles numbers in the list, converts them to string before concatenating. 
	# from https://stackoverflow.com/a/3590168/4355695
	print(line) # print to screen also
	f = open(logFolder + 'log.txt', 'a', newline='\r\n', encoding='utf8') #open in append mode
	print(timestamp, line, file=f)
	# `,file=f` argument at end writes the line, with newline as defined, to the file instead of to screen. 
	# from https://stackoverflow.com/a/2918367/4355695
	f.close()


###################################################           

# the __main__ function sets up the tornado server

if __name__ == "__main__":
    signal.signal(signal.SIGINT, signal_term_handler)
    app = make_app()
    portnum = 5010
    while True: # loop to increment the port number till we find one that isn't occupied
        try:
            port = int(os.environ.get("PORT", portnum))
            app.listen(port)
            break
        except OSError:
            portnum += 1
            if portnum > 9999: 
                logmessage('All ports from 5000 to 9999 seem busy, cannot run this program. Please check and retry.')
                sys.exit()

    thisURL = "http://localhost:" + str(port)
    webbrowser.open(thisURL)
    logmessage("Open " + thisURL + " in your Web Browser if you don't see it opening automatically within 5 seconds.")
    tornado.ioloop.IOLoop.current().start()
